@@ -115,32 +115,25 @@ def annlinit(model, abstol=1.0e-3, reltol=1.0e-3, nsteps = 1000, itermaxstep = N
     return [f, rhs_exprs, y, odesize, data, xout, yout, nsteps, cvode_mem, yzero, reltol, abstol], paramarray
 
 
-# reltol of 1.0e-3, relative error of ~1%. abstol of 1.0e-2, enough for values that oscillate in the hundreds to thousands
-def annlodesolve(model, tfinal, envlist, params, useparams=None, tinit = 0.0, ic=True):
+# reltol of 1.0e-3, relative error of ~1%. abstol of 1.0e-3, enough for values that oscillate in the hundreds to thousands
+def annlodesolve(model, tfinal, envlist, params, tinit = 0.0, ic=True):
     '''
     the ODE equation solver tailored to work with the annealing algorithm
     model: the model object
     envlist: the list returned from annlinit
     params: the list of parameters that are being optimized with annealing 
-    useparams: the parameter number to which params[i] corresponds
     tinit: initial time
     reltol: relative tolerance
     abstol: absolute tolerance
-    ic: reinitialize initial conditions to a value in params or useparams
+    ic: reinitialize initial conditions to a value in params 
     '''
     (f, rhs_exprs, y, odesize, data, xout, yout, nsteps, cvode_mem, yzero, reltol, abstol) = envlist
 
     #set the initial values and params in each run
-    #all parameters are used in annealing. initial conditions are not, here
-    if useparams is None:
-        for i in range(len(params)):
-            data.p[i] = params[i]
-    else:
-        #only a subset of parameters are used for annealing
-        for i in range(len(useparams)):
-            #print "changing parameter", model.parameters[useparams[i]],"data.p", data.p[useparams[i]],"to", params[i]
-            data.p[useparams[i]] = params[i]
-
+    #all parameters are used in annealing. 
+    for i in range(len(params)):
+        data.p[i] = params[i]
+        
     # update yzero if initial conditions are being modified as part of the parameters
     # did it this way b/c yzero and data.p may not always want to be modified at the same time
     # FIXME: this is not the best way to do this.
@@ -185,48 +178,19 @@ def annlodesolve(model, tfinal, envlist, params, useparams=None, tinit = 0.0, ic
     #print "Integration finished"
 
     #now deal with observables
-    obs_names = [name for name, rp in model.observable_patterns]
-    yobs = numpy.zeros([len(obs_names), nsteps])
+    yobs = numpy.zeros([len(model.observables), nsteps])
     
     #sum up the correct entities
-    for i, name in enumerate(obs_names):
-        factors, species = zip(*model.observable_groups[name])
-        yobs[i] = (yout[:, species] * factors).sum(1)
-
-    #transpose yobs to make it easy to plot
-    #yobs.T
+    for i, obs in enumerate(model.observables):
+        coeffs = obs.coefficients
+        specs  = obs.species
+        yobs[i] = (yout[:, specs] * coeffs).sum(1)
 
     #merge the x and y arrays for easy analysis
     xyobs = numpy.vstack((xout, yobs))
 
     return (xyobs,xout,yout, yobs)
 
-def read_csv_array(xpfname):
-    """returns the first string and a numpy array from a csv set of data
-    xpfname is a string with the file name"""
-
-    #get the experimental data needed for annealing
-    fp = open(xpfname, "r")
-
-    reader = csv.reader(fp)
-    templist = []
-    #read in the lists
-    for row in reader:
-        templist.append(row)
-    #remove empty spaces
-    for i in range(0, len(templist)):
-        templist[i] = filter(None, templist[i])
-    headstring = templist.pop(0)
-    #Now put these into a numpy array, assume they are all floats
-    converters = tuple([float]*len(templist[0]))
-    darray = numpy.zeros((len(templist), len(templist[0])))
-    for i, item in enumerate(templist):
-        darray[i] = numpy.asarray(templist[i], dtype=darray.dtype)    
-   
-    #transpose to ease analysis
-    darray = darray.T
-    return (darray, headstring)
-
 def compare_data(xparray, simarray, xspairlist, vardata=False):
     """Compares two arrays of different size and returns the X^2 between them.
     Uses the X axis as the unit to re-grid both arrays. 
@@ -257,8 +221,6 @@ def compare_data(xparray, simarray, xspairlist, vardata=False):
    
     for i in range(len(xspairlist)):
         # create a b-spline of the sim data and fit it to desired range
-        # import code
-        # code.interact(local=locals())
         
         #some error checking
         #print "xspairlist length:", len(xspairlist[i])
@@ -288,8 +250,7 @@ def compare_data(xparray, simarray, xspairlist, vardata=False):
         else:
         # assume a default variance
             xparrayvar = numpy.ones(xparray.shape[1])
-            xparrayvar = xparray[xparrayaxis]*.341 # 1 stdev w/in 1 sigma of the experimental data... 
-            xparrayvar = xparrayvar * xparrayvar
+            xparrayvar = xparray[xparrayaxis]*.25 #assume a coeff of variation of .25 = sigma/mean (from Chen, Gaudet...)
             # Remove any zeros in the variance array # FIXME
             for i in range(0, len(xparrayvar)):
                 if (xparrayvar[i] == 0):
@@ -306,104 +267,57 @@ def compare_data(xparray, simarray, xspairlist, vardata=False):
                 # print objarray
                 objarray[i] = 1e-100 #zero enough
 
-        #import code
-        #code.interact(local=locals())
-
         objout += objarray.sum()
         #print "OBJOUT(%d,%d):%f  |\t\tOBJOUT(CUM):%f"%(xparrayaxis, simarrayaxis, objarray.sum(), objout)
 
-    #print "OBJOUT(total):", objout
+    print "OBJOUT(total):", objout
     return objout
 
-def getgenparambounds(params, omag=1, N=1000., useparams=[], usemag=None, useN=None ):
-    # params must be a numpy array
-    # from: http://projects.scipy.org/scipy/ticket/1126
-    # The input-parameters "lower" and "upper" do not refer to global bounds of the
-    # parameter space but to 'maximum' displacements in the MC scheme AND
-    # in addition they determine the initial point!! 
-    # The initial value that you provide for the parameter vector seems to have no influence.
-    # This is how I call anneal with my desired functionality
-    # p=[a,b,c] #my initial values
-    # lb=array([a0,b0,c0]) #my lower bounds (absolute values)
-    # ub=array([a1,b1,c1]) #my upper bounds
-    # N=100 #determines the size of displacements; the more N, the smaller steps and the longer time to convergence
-    # dx=(ub-lb)/N #displacements--you could get from ub to lb in N steps
-    # lower=array(p)-dx/2 #the "lower bound" of the anneal routine (upper and lower step bounds relative to the current values of p)
-    # upper=array(p)+dx/2 #the "upper bound" of the anneal routine
-    # f=lambda var: costfunction(p,lb,ub) #my cost function that is made very high if not lb < p < ub
-    # pbest=scipy.optimize.anneal(f,p,lower=lower,upper=upper)
-    # This ensures a MC search that starts of close to my initial value and makes steps of dx in its search.
+def logparambounds(params, omag=1, useparams=[], usemag=None, initparams=[], initmag=None):
     ub = numpy.zeros(len(params))
     lb = numpy.zeros(len(params))
-    dx = numpy.zeros(len(params))
     # set upper/lower bounds for generic problem
     for i in range(len(params)):
-        if i in useparams:
+        if i in useparams and i not in initparams:
             ub[i] = params[i] * pow(10,usemag)
             lb[i] = params[i] / pow(10,usemag)
-            dx[i] = (ub[i] - lb[i])/useN
+        elif i in initparams:
+            ub[i] = params[i] * pow(10,initmag)
+            lb[i] = params[i] / pow(10,initmag)
         else:
             ub[i] = params[i] * pow(10, omag)
             lb[i] = params[i] / pow(10, omag)
-            dx[i] = (ub[i] - lb[i])/N
-    #print dx
-    lower = params - dx/2
-    lower[numpy.where(lower<0.)] = 0.0 #make sure we don't go negative on parameters
-    upper = params + dx/2
+    return lb, ub
 
-    return lb, ub, lower, upper
-
-def annealfxn(params, useparams, time, model, envlist, xpdata, xspairlist, lb, ub, norm=False, vardata=False, fileobj=None):
-    ''' Feeder function for scipy.optimize.anneal
-    '''
-    #annlout = scipy.optimize.anneal(pysb.anneal_sundials.annealfxn, paramarr, 
-    #                                args=(None, 20000, model, envlist, xpnormdata, 
-    #                                [(2,1),(4,2),(7,3)], lb, ub, True, True), 
-    #                                lower=lower, upper=upper, full_output=1)
-    # sample anneal call full model:
-    # params: parameters to be optimized, at their values for the given annealing step
-    # lower,upper: arrays from get array function or something similar from getgenparambounds
-    # lb, ub: lower bound and upper bound for function from getgenparambounds
-    #
-    # sample anneal call, optimization of some parameters
-    #   annlout = scipy.optimize.anneal(pysb.anneal_sundials.annealfxn, smacprm, args=(smacnum, 25000, model, envlist, xpdata,
-    #            [(2,2), (3,3)], lower=lower, upper=upper, full_output=1)
-    #
-    # sample anneal call, optimization for ALL parameters
-    # 
-    #
-
-    if numpy.greater_equal(params, lb).all() and numpy.less_equal(params, ub).all():
-        print "Integrating..."
-        outlist = annlodesolve(model, time, envlist, params, useparams)
-        # specify that this is normalized data
-        if norm is True:
-            print "Normalizing data"
-            datamax = numpy.max(outlist[0], axis = 1)
-            datamin = numpy.min(outlist[0], axis = 1)
-            outlistnorm = ((outlist[0].T - datamin)/(datamax-datamin)).T
-            # xpdata[0] should be time, get from original array
-            outlistnorm[0] = outlist[0][0].copy()
-            # xpdata here is normalized, and so is outlistnorm
-            objout = compare_data(xpdata, outlistnorm, xspairlist, vardata)
+def linparambounds(params, fact=.25, useparams=[], usefact=None):
+    ub = numpy.zeros(len(params))
+    lb = numpy.zeros(len(params))
+    # set upper/lower bounds for generic problem
+    for i in range(len(params)):
+        if i in useparams:
+            ub[i] = params[i] + (params[i] * fact)
+            lb[i] = params[i] - (params[i] * fact)
         else:
-            objout = compare_data(xpdata, outlist[0], xspairlist, vardata)
-    else:
-        print "======>VALUE OUT OF BOUNDS NOTED"
-        temp = numpy.where((numpy.logical_and(numpy.greater_equal(params, lb), numpy.less_equal(params, ub)) * 1) == 0)
-        for i in temp:
-            print "======>",i, params[i]
-        objout = 1.0e300 # the largest FP in python is 1.0e308, otherwise it is just Inf
+            ub[i] = params[i] + (params[i] * usefact)
+            lb[i] = params[i] - (params[i] * usefact)
+    lb[numpy.where(lower<0.)] = 0.0 #make sure we don't go negative on parameters...
+    return lb, ub
 
-    # save the params and temps for analysis
-    # FIXME If a parameter is out of bounds, outlist and outlistnorm will be undefined and this will cause an error
-    if fileobj:
-        if norm:
-            writetofile(fileobj, params, outlistnorm, objout)
-        else:
-            writetofile(fileobj, params, outlist, objout)
+def mapprms(nums01, lb, ub, scaletype="log"):
+    """given an upper bound(ub), lower bound(lb), and a sample between zero and one (zosample)
+    return a set of parameters within the lb, ub range. 
+    nums01: array of numbers between zero and 1
+    lb: array of lower bound for each parameter
+    ub: arary of upper bound for each parameter
+    """
+    params = numpy.zeros_like(nums01)
     
-    return objout
+    if scaletype == "log":
+        params = lb*(ub/lb)**nums01 # map the [0..1] array to values sampled over their omags
+    elif scaletype == "lin":
+        params = (nums01*(ub-lb)) + lb
+
+    return params
 
 def writetofile(fout, simparms, simdata, temperature):
     imax, jmax = simdata.shape
@@ -430,9 +344,6 @@ def writetofile(fout, simparms, simdata, temperature):
     fout.write('#-------------------------------------------------------------------------------------------------\n')
     return
 
-# FIXME
-# FIXME: THESE FUNCTIONS SHOULD PROBABLY NOT BE INCLUDED IN THE FINAL VERSION OF THE ANNEAL SUNDIALS FUNCTION
-# FIXME
 
 def tenninetycomp(outlistnorm, arglist, xpsamples=1.0):
     """ Determine Td and Ts. Td calculated at time when signal goes up to 10%.
@@ -469,28 +380,43 @@ def tenninetycomp(outlistnorm, arglist, xpsamples=1.0):
     #
     
     obj = ((1./varTdxp) * (Tdsim - Tdxp)**2.) + ((1./varTsxp) * (Tssim - Tsxp)**2.)
-    obj *= xpsamples
+    #obj *= xpsamples
     
-    print "OBJOUT-10-90:(%f,%f):%f"%(Tdsim, Tssim, obj)
+    print "OBJOUT-10-90:(%g,%g):%g"%(Tdsim, Tssim, obj)
 
     return obj    
 
+def annealfxn(zoparams, time, model, envlist, xpdata, xspairlist, lb, ub, tn = [], scaletype="log", norm=True, vardata=False, fileobj=None):
+    """Feeder function for scipy.optimize.anneal
+    zoparams: the parameters in the range [0,1) to be sampled
+    time: the time scale for the simulation
+    model: a PySB model object
+    envlist: an environment list for the sundials integrator
+    xpdata: experimental data
+    xspairlist: the pairlist of the correspondence of experimental and simulation outputs
+    lb: lower bound for parameters
+    ub: upper bound for parameters
+    tn: list of values for ten-ninety fits
+    scaletype: log, linear,etc to convert zoparams to real params b/w lb and ub. default "log"
+    norm: normalization on. default true
+    vardata: variance data available. default "false"
+    fileobj: file object to write data output. default "None"
 
-def annealfxncust(params, useparams, time, model, envlist, xpdata, xspairlist, tenninetylist, lb, ub, norm=False, vardata=False, fileobj = False):
-    ''' Feeder function for scipy.optimize.anneal
-    '''
-    # Customized anneal function for the case when Smac is not fit to a function. Here we
-    # measure the Smac output from the model and use a 10-90 criterion to extract Td and Ts.
-    # We then use a chi-square comparison to these two values for the Smac contribution to the 
-    # data fitting. 
-    #
-    # 
-    #
+    """
 
-    if numpy.greater_equal(params, lb).all() and numpy.less_equal(params, ub).all():
-        print "Integrating..."
-        outlist = annlodesolve(model, time, envlist, params, useparams)
-        # specify that this is normalized data
+    # convert of linear values from [0,1) to desired sampling distrib
+    paramarr = mapprms(zoparams, lb, ub, scaletype="log")
+
+    #debug
+    #print "ZOPARAMS:\n", zoparams,"\n"
+    #print paramarr
+
+    # eliminate values outside the boundaries, i.e. those outside [0,1)
+    if numpy.greater_equal(paramarr, lb).all() and numpy.less_equal(paramarr, ub).all():
+        print "integrating... "
+        outlist = annlodesolve(model, time, envlist, paramarr)
+
+        # normalized data needs a bit more tweaking before objfxn calculation
         if norm is True:
             print "Normalizing data"
             datamax = numpy.max(outlist[0], axis = 1)
@@ -498,34 +424,34 @@ def annealfxncust(params, useparams, time, model, envlist, xpdata, xspairlist, t
             outlistnorm = ((outlist[0].T - datamin)/(datamax-datamin)).T
             # xpdata[0] should be time, get from original array
             outlistnorm[0] = outlist[0][0].copy()
-            # xpdata here is normalized, and so is outlistnorm
+            # xpdata here should be normalized
             objout = compare_data(xpdata, outlistnorm, xspairlist, vardata)
-            # This takes care of the IC/EC-RP comparisons
-            # Now SMAC
-            tn = tenninetycomp(outlistnorm, tenninetylist,len(xpdata[0]))
-            objout += tn 
-            print "objout TOT:", objout
+            if tn:
+                tn = tenninetycomp(outlistnorm, tn, len(xpdata[0]))
+                objout += tn 
+            print "NORM objout TOT:", objout
         else:
             objout = compare_data(xpdata, outlist[0], xspairlist, vardata)
-            tn = tenninetycomp(outlistnorm, tenninetylist)
-            objout += tn 
+            if tn:
+                tn = tenninetycomp(outlist[0], tn)
+                objout += tn 
+            print "objout TOT:", objout
     else:
-        print "======>VALUE OUT OF BOUNDS NOTED"
-        temp = numpy.where((numpy.logical_and(numpy.greater_equal(params, lb), numpy.less_equal(params, ub)) * 1) == 0)
+        print "======> VALUE OUT OF BOUNDS NOTED"
+        temp = numpy.where((numpy.logical_and(numpy.greater_equal(paramarr, lb), numpy.less_equal(paramarr, ub)) * 1) == 0)
         for i in temp:
-            print "======>",i, params[i]
+            print "======>",i,"\n======", paramarr[i],"\n", zoparams[i],"\n"
         objout = 1.0e300 # the largest FP in python is 1.0e308, otherwise it is just Inf
+
+    # save the params and temps for analysis
+
+    # FIXME If a parameter is out of bounds, outlist and outlistnorm will be undefined and this will cause an error
+    if fileobj:
+        if norm:
+            writetofile(fileobj, paramarr, outlistnorm, objout)
+        else:
+            writetofile(fileobj, paramarr, outlist, objout)
+    
     return objout
 
-    
-
-
-
-
-
-    
-
-
-
-
 
